#!/bin/tclsh

# path to user dirs
set userpath         "/ftp-data/users/"
# users to ignore
set ignore           "glftpd default.user"
# default group if a guy isn't in a group!
set default_group    "GROUPLESS"
# the sections.. format: "<SECTION NAME>:<STAT NUMBER> [..]"
set sections         "CORE:0 MP3:1 SERIES:2"
# are you using this as a glftpd command?
set glftpd_command   1


proc usage { } {
  puts stdout " +----------------------------------------------------+"
  puts stdout " | fsUStats v1.0 - coded by fusion (fs2oo2@yahoo.com) |"
  puts stdout " +----------------------------------------------------+"
  puts stdout " | USAGE: fsUStats.tcl -\[dwma\] -\[ul\] \[-s<#>\] \[-f\] \[#\] |"
  puts stdout " |         -\[dwma\]   - d:day, w:week, m:month, a:all  |"
  puts stdout " |         -\[ul\]     - u: upload, l: download         |"
  puts stdout " |         -s<#>     - section number                 |"
  puts stdout " |         -f        - sort by number of files        |"
  puts stdout " |         \[#\]       - number of users to show        |"
  puts stdout " +----------------------------------------------------+"
  exit
}

fconfigure stdout -buffering none

if { $argv == "" } { usage }

set arg_period  ""
set arg_type    ""
set arg_users   10
set arg_section 0
set arg_sort    1

while { [string index $argv 0] == "-" } {
  switch -- [string tolower [string index $argv 1]] {
    d { set arg_period "day" }
    w { set arg_period "week" }
    m { set arg_period "month" }
    a { set arg_period "all" }
    u { set arg_type "up" }
    l { set arg_type "down" }
    s {
      if { [string is integer [string index $argv 2]] } {
        set arg_section [string index $argv 2]
      }
    }
    f { set arg_sort 2 }
  }
  set argv [lrange $argv 1 end]
}

if { ! [string is integer [lindex $argv 0]] || $arg_period == "" || $arg_type == "" } { usage }

if { $glftpd_command == 1 } {
  if { [info exists env(SECTION)] } {
    set i [lsearch -glob $sections "$env(SECTION):*"]
    if { $i != -1 } {
      set arg_section [lindex [split [lindex $sections $i] ":"] end]
    }
  }
}

if { [lindex $argv 0] != "" } {
  set arg_users [lindex $argv 0]
  if { $arg_users < 1 } { usage }
}

set user(count) 0
set sort        ""
set index_megs  [expr 2 + 3 * $arg_section]
set index_files [expr 2 + 3 * $arg_section - 1]

foreach file [glob -nocomplain -- "$userpath*"] {
  set username [lindex [split $file "/"] end]
  if { [lsearch -exact $ignore $username] == -1 && [string index $username end] != "~" && ! [catch { set handle [open $file "r"] }] } {
    foreach line [split [read $handle] "\n"] {
      switch -- [lindex $line 0] {
        DAYUP {
          if { $arg_period == "day" && $arg_type == "up" } {
            lappend sort "$username [lindex $line $index_megs] [lindex $line $index_files]"
          }
        }
        DAYDN {
          if { $arg_period == "day" && $arg_type == "down" } {
            lappend sort "$username [lindex $line $index_megs] [lindex $line $index_files]"
          }
        }
        WKUP {
          if { $arg_period == "week" && $arg_type == "up" } {
            lappend sort "$username [lindex $line $index_megs] [lindex $line $index_files]"
          }
        }
        WKDN {
          if { $arg_period == "week" && $arg_type == "down" } {
            lappend sort "$username [lindex $line $index_megs] [lindex $line $index_files]"
          }
        }
        MONTHUP {
          if { $arg_period == "month" && $arg_type == "up" } {
            lappend sort "$username [lindex $line $index_megs] [lindex $line $index_files]"
          }
        }
        MONTHDN {
          if { $arg_period == "month" && $arg_type == "down" } {
            lappend sort "$username [lindex $line $index_megs] [lindex $line $index_files]"
          }
        }
        ALLUP {
          if { $arg_period == "all" && $arg_type == "up" } {
            lappend sort "$username [lindex $line $index_megs] [lindex $line $index_files]"
          }
        }
        ALLDN {
          if { $arg_period == "all" && $arg_type == "down" } {
            lappend sort "$username [lindex $line $index_megs] [lindex $line $index_files]"
          }
        }
        GROUP {
          if { ! [info exists user($username,group)] } {
            set user($username,group) [lindex $line 1]
          }
        }
        TAGLINE {
          set user($username,tagline) [lrange $line 1 end]
        }
        default { }
      }
    }
    if { ! [info exists user($username,group)] } {
      set user($username,group) $default_group
    }
    close $handle
    incr user(count)
  }
}

set sort [lsort -decreasing -integer -index $arg_sort $sort]

set i 1

puts stdout "  .-------------------------------------------------------------------------------."
puts stdout "  |  pos  username       group      tagline                       bytes    files  |"
puts stdout "  |-------------------------------------------------------------------------------|"

if { $arg_users > $user(count) } {
  set arg_users $user(count)
}

while { $i <= $arg_users } {
  set info [lindex $sort [expr $i - 1]]
  set username  [lindex $info 0]
  set group $user($username,group)
  set tagline [string range $user($username,tagline) 0 24]
  set bytes [lindex $info 1]
  set files [lindex $info 2]
  set unit "KB"
  set number $i
  while { $bytes > 1024 } {
    set bytes [expr $bytes / 1024.00]
    switch -- $unit {
      KB { set unit "MB" }
      MB { set unit "GB" }
      GB { set unit "TB" }
      default { set unit "??" }
    }
  }
  set tmp [split $bytes "."]
  set s [string range [lindex $tmp 1] 0 1]
  while { [string length $s] < 2 } { set s "${s}0" }
  set bytes "[lindex $tmp 0].$s"
  while { [string length $number] < 3 } { set number "0$number" }
  while { [string length $username] < 13 } { set username "$username " }
  while { [string length $group] < 9 } { set group "$group " }
  while { [string length $bytes] < 6 } { set bytes " $bytes" }
  while { [string length $files] < 7 } { set files " $files" }
  while { [string length $tagline] < 25 } { set tagline "$tagline " }
  puts stdout "  |  $number  $username  $group  $tagline  $bytes$unit  $files  |"
  incr i
}
puts stdout "  `--------------------------------------------------------------------<fsUStats>-'"


